--==============================================================================
-- CERN (BE-CO-HT)
-- CONV-TTL-RS485 PTS top-level file
--==============================================================================
--
-- author: Theodor Stana (t.stana@cern.ch)
--
-- date of creation: 2014-10-30
--
-- version: 1.0
--
-- description:
--
-- dependencies:
--
-- references:
--
--==============================================================================
-- GNU LESSER GENERAL PUBLIC LICENSE
--==============================================================================
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--==============================================================================
-- last changes:
--    2014-10-30   Theodor Stana     File created
--==============================================================================
-- TODO: -
--==============================================================================

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.conv_common_gw_pkg.all;

use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.genram_pkg.all;

entity pts is
  port
  (
    -- Clocks
    clk_20_i                : in  std_logic;
    clk_125_p_i             : in  std_logic;
    clk_125_n_i             : in  std_logic;

    -- I2C interface
    scl_i                   : in  std_logic;
    scl_o                   : out std_logic;
    scl_en_o                : out std_logic;
    sda_i                   : in  std_logic;
    sda_o                   : out std_logic;
    sda_en_o                : out std_logic;

    -- VME interface
    vme_sysreset_n_i        : in  std_logic;
    vme_ga_i                : in  std_logic_vector(4 downto 0);
    vme_gap_i               : in  std_logic;

    -- Channel enable
    global_oen_o           : out std_logic;
    ttl_oen_o              : out std_logic;
    inv_oen_o              : out std_logic;
    rs485_oen_o            : out std_logic;

    -- Front panel channels
    ttl_n_i                 : in  std_logic_vector(5 downto 0);
    ttl_o                   : out std_logic_vector(5 downto 0);
    inv_n_i                 : in  std_logic_vector(3 downto 0);
    inv_o                   : out std_logic_vector(3 downto 0);

    -- Rear panel channels
    rs485_n_i               : in  std_logic_vector(5 downto 0);
    rs485_fs_n_i            : in  std_logic_vector(5 downto 0);
    rs485_o                 : out std_logic_vector(5 downto 0);

    -- Channel leds
    led_front_n_o           : out std_logic_vector(5 downto 0);
    led_rear_n_o            : out std_logic_vector(5 downto 0);

    -- SPI interface to on-board flash chip
    flash_cs_n_o            : out std_logic;
    flash_sclk_o            : out std_logic;
    flash_mosi_o            : out std_logic;
    flash_miso_i            : in  std_logic;

    -- PLL DACs
    -- 20 MHz VCXO control
    dac20_din_o             : out std_logic;
    dac20_sclk_o            : out std_logic;
    dac20_sync_n_o          : out std_logic;
    -- 125 MHz clock generator control
    dac125_din_o            : out std_logic;
    dac125_sclk_o           : out std_logic;
    dac125_sync_n_o         : out std_logic;

    -- SFP lines
    sfp_los_i               : in    std_logic;
    sfp_mod_def0_i          : in    std_logic;
    sfp_rate_select_o       : out   std_logic;
    sfp_mod_def1_b          : inout std_logic;
    sfp_mod_def2_b          : inout std_logic;
    sfp_tx_disable_o        : out   std_logic;
    sfp_tx_fault_i          : in    std_logic;

    -- Thermometer data port
    thermometer_b           : inout std_logic;

    -- Switches
    sw_gp_n_i               : in   std_logic_vector(7 downto 0);
    sw_multicast_n_i        : in   std_logic_vector(3 downto 0);

    -- RTM lines
    rtmm_i                  : in  std_logic_vector(2 downto 0);
    rtmp_i                  : in  std_logic_vector(2 downto 0);

    -- Front panel bicolor LEDs
    led_ctrl0_o             : out std_logic;
    led_ctrl0_oen_o         : out std_logic;
    led_ctrl1_o             : out std_logic;
    led_ctrl1_oen_o         : out std_logic;
    led_gp_2_4_o            : out std_logic;
    led_gp_1_3_o            : out std_logic;
    led_oterm_wr_o          : out std_logic;
    led_iterm_syserror_o    : out std_logic;
    led_gf_syspw_o          : out std_logic;
    led_ttl_i2c_o           : out std_logic
  );
end entity pts;


architecture arch of pts is

  --============================================================================
  -- Type declarations
  --============================================================================

  --============================================================================
  -- Constant declarations
  --============================================================================
  -- Board ID constant
  constant c_board_id         : std_logic_vector(31 downto 0) := x"54343835";
  -- Number of Wishbone masters and slaves, for wb_crossbar
  constant c_nr_masters       : natural :=  1;
  constant c_nr_slaves        : natural :=  1;

  -- slave order definitions
  constant c_slv_pts_regs      : natural :=  0;
  constant c_slv_onewire_mst   : natural :=  1;
  constant c_slv_dac_spi_125   : natural :=  2;
  constant c_slv_dac_spi_20    : natural :=  3;
  constant c_slv_clk_info_125  : natural :=  4;
  constant c_slv_clk_info_20   : natural :=  5;
  constant c_slv_sfp_i2c       : natural :=  6;
  constant c_slv_minic         : natural :=  7;
  constant c_slv_endpoint      : natural :=  8;
  constant c_slv_dpram         : natural :=  9;
  constant c_slv_pulse_cntrs   : natural := 10;

  -- base address definitions
  constant c_addr_pts_regs     : t_wishbone_address := x"00000000";
  constant c_addr_onewire_mst  : t_wishbone_address := x"00000010";
  constant c_addr_dac_spi_125  : t_wishbone_address := x"00000020";
  constant c_addr_dac_spi_20   : t_wishbone_address := x"00000080";
  constant c_addr_clk_info_125 : t_wishbone_address := x"00000100";
  constant c_addr_clk_info_20  : t_wishbone_address := x"00000120";
  constant c_addr_sfp_i2c      : t_wishbone_address := x"00000140";
  constant c_addr_endpoint     : t_wishbone_address := x"00000200";
  constant c_addr_minic        : t_wishbone_address := x"00000400";
  constant c_addr_dpram        : t_wishbone_address := x"00000800";
  constant c_addr_pulse_cntrs  : t_wishbone_address := x"00000C00";

  -- address mask definitions
  constant c_mask_pts_regs     : t_wishbone_address := x"00000FF0";
  constant c_mask_onewire_mst  : t_wishbone_address := x"00000FF0";
  constant c_mask_dac_spi_125  : t_wishbone_address := x"00000FE0";
  constant c_mask_dac_spi_20   : t_wishbone_address := x"00000FE0";
  constant c_mask_clk_info_125 : t_wishbone_address := x"00000F60";
  constant c_mask_clk_info_20  : t_wishbone_address := x"00000F60";
  constant c_mask_sfp_i2c      : t_wishbone_address := x"00000F60";
  constant c_mask_endpoint     : t_wishbone_address := x"00000E00";
  constant c_mask_minic        : t_wishbone_address := x"00000C00";
  constant c_mask_dpram        : t_wishbone_address := x"00000C00";
  constant c_mask_pulse_cntrs  : t_wishbone_address := x"00000C00";

  -- addresses constant for Wishbone crossbar
  constant c_addresses  : t_wishbone_address_array(c_nr_slaves-1 downto 0)
                        := (
                             c_slv_pts_regs     => c_addr_pts_regs
--                             c_slv_onewire_mst  => c_addr_onewire_mst,
--                             c_slv_dac_spi_125  => c_addr_dac_spi_125,
--                             c_slv_clk_info_125 => c_addr_clk_info_125,
--                             c_slv_dac_spi_20   => c_addr_dac_spi_20,
--                             c_slv_clk_info_20  => c_addr_clk_info_20,
--                             c_slv_sfp_i2c      => c_addr_sfp_i2c,
--                             c_slv_endpoint     => c_addr_endpoint,
--                             c_slv_minic        => c_addr_minic,
--                             c_slv_dpram        => c_addr_dpram,
--                             c_slv_pulse_cntrs  => c_addr_pulse_cntrs
                           );

  -- masks constant for Wishbone crossbar
  constant c_masks      : t_wishbone_address_array(c_nr_slaves-1 downto 0)
                        := (
                             c_slv_pts_regs     => c_mask_pts_regs
--                             c_slv_onewire_mst  => c_mask_onewire_mst,
--                             c_slv_dac_spi_125  => c_mask_dac_spi_125,
--                             c_slv_clk_info_125 => c_mask_clk_info_125,
--                             c_slv_dac_spi_20   => c_mask_dac_spi_20,
--                             c_slv_clk_info_20  => c_mask_clk_info_20,
--                             c_slv_sfp_i2c      => c_mask_sfp_i2c,
--                             c_slv_endpoint     => c_mask_endpoint,
--                             c_slv_minic        => c_mask_minic,
--                             c_slv_dpram        => c_mask_dpram,
--                             c_slv_pulse_cntrs  => c_mask_pulse_cntrs
                           );

  -- MiniNIC log2 of memory size
  constant c_minic_memsize_log2 : natural := 10;

  --============================================================================
  -- Component declarations
  --============================================================================
  -- Regs to test I2C operation
  component pts_regs is
    port (
      rst_n_i                                  : in     std_logic;
      clk_sys_i                                : in     std_logic;
      wb_adr_i                                 : in     std_logic_vector(0 downto 0);
      wb_dat_i                                 : in     std_logic_vector(31 downto 0);
      wb_dat_o                                 : out    std_logic_vector(31 downto 0);
      wb_cyc_i                                 : in     std_logic;
      wb_sel_i                                 : in     std_logic_vector(3 downto 0);
      wb_stb_i                                 : in     std_logic;
      wb_we_i                                  : in     std_logic;
      wb_ack_o                                 : out    std_logic;
      wb_stall_o                               : out    std_logic;
  -- Port for std_logic_vector field: 'ID register bits' in reg: 'BIDR'
      pts_bidr_i                               : in     std_logic_vector(31 downto 0);
  -- Port for BIT field: 'Front pulse LED enable' in reg: 'CSR'
      pts_csr_front_led_en_o                   : out    std_logic;
  -- Port for BIT field: 'Rear pulse LED enable' in reg: 'CSR'
      pts_csr_rear_led_en_o                    : out    std_logic;
  -- Port for BIT field: 'Status LED enable' in reg: 'CSR'
      pts_csr_stat_led_en_o                    : out    std_logic;
  -- Port for BIT field: 'TTL pulse enable' in reg: 'CSR'
      pts_csr_ttl_en_o                         : out    std_logic;
  -- Port for BIT field: 'Blocking pulse enable' in reg: 'CSR'
      pts_csr_blo_en_o                         : out    std_logic;
  -- Port for BIT field: 'Blocking LED control' in reg: 'CSR'
      pts_csr_blo_led_o                        : out    std_logic;
  -- Port for BIT field: 'reset' in reg: 'CSR'
      pts_csr_rst_o                            : out    std_logic;
  -- Port for std_logic_vector field: 'switches' in reg: 'CSR'
      pts_csr_switch_i                         : in     std_logic_vector(7 downto 0);
  -- Port for std_logic_vector field: 'RTM' in reg: 'CSR'
      pts_csr_rtm_i                            : in     std_logic_vector(5 downto 0);
  -- Ports for BIT field: 'I2C communication error' in reg: 'CSR'
      pts_csr_i2c_err_o                        : out    std_logic;
      pts_csr_i2c_err_i                        : in     std_logic;
      pts_csr_i2c_err_load_o                   : out    std_logic;
  -- Ports for BIT field: 'I2C communication watchdog timeout error' in reg: 'CSR'
      pts_csr_i2c_wdto_o                       : out    std_logic;
      pts_csr_i2c_wdto_i                       : in     std_logic;
      pts_csr_i2c_wdto_load_o                  : out    std_logic
    );
  end component pts_regs;

  --============================================================================
  -- Signal declarations
  --============================================================================
  -- Reset signals
  signal rst_20_n            : std_logic;
  signal rst_125_n           : std_logic;
  signal rst_fr_reg          : std_logic;
  signal rst_ext             : std_logic;

  -- I2C bridge signals
  signal i2c_addr            : std_logic_vector(6 downto 0);
  signal i2c_tip             : std_logic;
  signal i2c_err_p           : std_logic;
  signal i2c_wdto_p          : std_logic;

  -- Wishbone crossbar signals
  signal xbar_slave_in       : t_wishbone_slave_in_array(c_nr_masters-1 downto 0);
  signal xbar_slave_out      : t_wishbone_slave_out_array(c_nr_masters-1 downto 0);
  signal xbar_master_in      : t_wishbone_master_in_array(c_nr_slaves-1 downto 0);
  signal xbar_master_out     : t_wishbone_master_out_array(c_nr_slaves-1 downto 0);

  -- LED signals
  signal cnt_halfsec         : unsigned(23 downto 0);
  signal led_seq             : unsigned(4 downto 0);
  signal pulse_led_en        : std_logic;
  signal stat_led_en         : std_logic;
  signal pulse_led_en_d0     : std_logic;
  signal stat_led_en_d0      : std_logic;
  signal pulse_led_en_risedge: std_logic;
  signal stat_led_en_risedge : std_logic;
  signal pulse_led_seq       : std_logic_vector(5 downto 0);
  signal bicolor_led_state   : std_logic_vector(23 downto 0);

  -- Signals to/from PTS regs component
  signal i2c_wdto_bit        : std_logic;
  signal i2c_wdto_bit_rst    : std_logic;
  signal i2c_wdto_bit_rst_ld : std_logic;
  signal i2c_err_bit         : std_logic;
  signal i2c_err_bit_rst     : std_logic;
  signal i2c_err_bit_rst_ld  : std_logic;
  signal rtm_lines           : std_logic_vector(5 downto 0);
  signal switches            : std_logic_vector(7 downto 0);

--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- Internal reset generation
  --============================================================================
  -- External reset input to reset generator
  rst_ext <= rst_fr_reg or (not vme_sysreset_n_i);

  -- Configure reset generator for 100ms reset
  cmp_reset_gen : conv_reset_gen
    generic map
    (
      -- Reset time: 50ns * 2 * (10**6) = 100 ms
      g_reset_time => 2*(10**6)
    )
    port map
    (
      clk_i   => clk_20_i,
      rst_i   => rst_ext,
      rst_n_o => rst_20_n
    );

  --============================================================================
  -- I2C bridge logic
  --============================================================================
  -- Set the I2C address signal according to ELMA protocol [2]
  i2c_addr <= "10" & vme_ga_i;

  -- Instantiate I2C bridge component
  --
  -- FSM watchdog timeout timer:
  -- * consider bit period of 30 us
  -- * 10 bits / byte transfer => 300 us
  -- * 40 bytes in one transfer => 12000 us
  -- * clk_i period = 50 ns => g_fsm_wdt = 12000 us / 50 ns = 240000
  -- * multiply by two for extra safety => g_fsm_wdt = 480000
  -- * Time to watchdog timeout: 480000 * 50ns = 24 ms
  cmp_i2c_bridge : wb_i2c_bridge
    generic map
    (
      g_fsm_wdt => 480000
    )
    port map
    (
      -- Clock, reset
      clk_i      => clk_20_i,
      rst_n_i    => rst_20_n,

      -- I2C lines
      scl_i      => scl_i,
      scl_o      => scl_o,
      scl_en_o   => scl_en_o,
      sda_i      => sda_i,
      sda_o      => sda_o,
      sda_en_o   => sda_en_o,

      -- I2C address and status
      i2c_addr_i => i2c_addr,

      -- TIP and ERR outputs
      tip_o      => i2c_tip,
      err_p_o    => i2c_err_p,
      wdto_p_o   => i2c_wdto_p,

      -- Wishbone master signals
      wbm_stb_o  => xbar_slave_in(0).stb,
      wbm_cyc_o  => xbar_slave_in(0).cyc,
      wbm_sel_o  => xbar_slave_in(0).sel,
      wbm_we_o   => xbar_slave_in(0).we,
      wbm_dat_i  => xbar_slave_out(0).dat,
      wbm_dat_o  => xbar_slave_in(0).dat,
      wbm_adr_o  => xbar_slave_in(0).adr,
      wbm_ack_i  => xbar_slave_out(0).ack,
      wbm_rty_i  => xbar_slave_out(0).rty,
      wbm_err_i  => xbar_slave_out(0).err
    );

  -- Register for the I2C_WDTO bit in the SR, cleared by writing a '1'
  p_sr_wdto_bit : process(clk_20_i)
  begin
    if rising_edge(clk_20_i) then
      if (rst_20_n = '0') then
        i2c_wdto_bit <= '0';
      elsif (i2c_wdto_p = '1') then
        i2c_wdto_bit <= '1';
      elsif (i2c_wdto_bit_rst_ld = '1') and (i2c_wdto_bit_rst = '1') then
        i2c_wdto_bit <= '0';
      end if;
    end if;
  end process p_sr_wdto_bit;

  -- Register for the I2C_ERR bit in the SR
  p_i2c_err_led : process(clk_20_i)
  begin
    if rising_edge(clk_20_i) then
      if (rst_20_n = '0') then
        i2c_err_bit <= '0';
      elsif (i2c_err_p = '1') then
        i2c_err_bit <= '1';
      elsif (i2c_err_bit_rst_ld = '1') and (i2c_err_bit_rst = '1') then
        i2c_err_bit <= '0';
      end if;
    end if;
  end process p_i2c_err_led;

  --============================================================================
  -- Instantiation and connection of the main Wishbone crossbar
  --============================================================================
  cmp_wb_crossbar : xwb_crossbar
    generic map
    (
      g_num_masters => c_nr_masters,
      g_num_slaves  => c_nr_slaves,
      g_registered  => false,
      g_address     => c_addresses,
      g_mask        => c_masks
    )
    port map
    (
      clk_sys_i => clk_20_i,
      rst_n_i   => rst_20_n,
      slave_i   => xbar_slave_in,
      slave_o   => xbar_slave_out,
      master_i  => xbar_master_in,
      master_o  => xbar_master_out
    );

  --============================================================================
  -- PTS registers
  --============================================================================
  -- RTM lines combo
  rtm_lines <= rtmp_i & rtmm_i;
  switches  <= not sw_gp_n_i;

  -- Regs to test I2C operation
  cmp_pts_regs : pts_regs
    port map
    (
      rst_n_i        => rst_20_n,
      clk_sys_i      => clk_20_i,
      wb_adr_i       => xbar_master_out(c_slv_pts_regs).adr(2 downto 2),
      wb_dat_i       => xbar_master_out(c_slv_pts_regs).dat,
      wb_dat_o       => xbar_master_in(c_slv_pts_regs).dat,
      wb_cyc_i       => xbar_master_out(c_slv_pts_regs).cyc,
      wb_sel_i       => xbar_master_out(c_slv_pts_regs).sel,
      wb_stb_i       => xbar_master_out(c_slv_pts_regs).stb,
      wb_we_i        => xbar_master_out(c_slv_pts_regs).we,
      wb_ack_o       => xbar_master_in(c_slv_pts_regs).ack,
      wb_stall_o     => xbar_master_in(c_slv_pts_regs).stall,

      -- Board ID register
      pts_bidr_i     => c_board_id,

      -- PTS control register
      pts_csr_front_led_en_o  => pulse_led_en,
      pts_csr_rear_led_en_o   => open,
      pts_csr_stat_led_en_o   => stat_led_en,
      pts_csr_ttl_en_o        => open,
      pts_csr_blo_en_o        => open,
      pts_csr_blo_led_o       => open,
      pts_csr_rst_o           => rst_fr_reg,
      pts_csr_switch_i        => switches,
      pts_csr_rtm_i           => rtm_lines,
      pts_csr_i2c_err_o       => i2c_err_bit_rst,
      pts_csr_i2c_err_i       => i2c_err_bit,
      pts_csr_i2c_err_load_o  => i2c_err_bit_rst_ld,
      pts_csr_i2c_wdto_o      => i2c_wdto_bit_rst,
      pts_csr_i2c_wdto_i      => i2c_wdto_bit,
      pts_csr_i2c_wdto_load_o => i2c_wdto_bit_rst_ld
    );

  --============================================================================
  -- LED test logic
  -- * test bicolor LEDs and its driving circuit (IC1)
  -- * test front panel LED logic and driving circuit (IC5)
  --============================================================================
  -- Rising edge detector for pulse LED enable signal
  p_pulse_led_en_risedge : process (clk_20_i)
  begin
    if rising_edge(clk_20_i) then
      if (rst_20_n = '0') then
        pulse_led_en_d0      <= '0';
        pulse_led_en_risedge <= '0';
      else
        pulse_led_en_d0      <= pulse_led_en;
        pulse_led_en_risedge <= '0';
        if (pulse_led_en_d0 = '0') and (pulse_led_en = '1') then
          pulse_led_en_risedge <= '1';
        end if;
      end if;
    end if;
  end process p_pulse_led_en_risedge;

  -- Rising edge detector for status LED enable signal
  p_stat_led_en_risedge : process (clk_20_i)
  begin
    if rising_edge(clk_20_i) then
      if (rst_20_n = '0') then
        stat_led_en_d0      <= '0';
        stat_led_en_risedge <= '0';
      else
        stat_led_en_d0      <= stat_led_en;
        stat_led_en_risedge <= '0';
        if (stat_led_en_d0 = '0') and (stat_led_en = '1') then
          stat_led_en_risedge <= '1';
        end if;
      end if;
    end if;
  end process p_stat_led_en_risedge;

  -- Process to control the LED sequence counter

  p_led_seq : process (clk_20_i) is
  begin
    if rising_edge(clk_20_i) then
      if (rst_20_n = '0') or (stat_led_en_risedge = '1')
                          or (pulse_led_en_risedge = '1') then
        cnt_halfsec <= (others => '0');
        led_seq     <= (others => '0');
      elsif (pulse_led_en = '1') or (stat_led_en = '1') then
        cnt_halfsec <= cnt_halfsec + 1;
        if (cnt_halfsec = 9999999) then
          cnt_halfsec <= (others => '0');
          led_seq     <= led_seq + 1;
          if (led_seq = 24) then
            led_seq   <= (others => '0');
          end if;
        end if;
      end if;
    end if;
  end process p_led_seq;

  -- Light each LED red and green in a sequence, based on the sequence counter.
  --
  -- The colors are set via the LED state vector (two bits per LED) as follows:
  --    state(1..0)     color
  --        00           OFF
  --        01           RED
  --        10          GREEN
  bicolor_led_state <= "000000000001000000000000" when (stat_led_en = '1') and (led_seq =  1) else
                       "000000000100000000000000" when (stat_led_en = '1') and (led_seq =  2) else
                       "000000010000000000000000" when (stat_led_en = '1') and (led_seq =  3) else
                       "000001000000000000000000" when (stat_led_en = '1') and (led_seq =  4) else
                       "000000000000000001000000" when (stat_led_en = '1') and (led_seq =  5) else
                       "000000000000000000010000" when (stat_led_en = '1') and (led_seq =  6) else
                       "000000000000000000000100" when (stat_led_en = '1') and (led_seq =  7) else
                       "000000000000000000000001" when (stat_led_en = '1') and (led_seq =  8) else
                       "000000000000000100000000" when (stat_led_en = '1') and (led_seq =  9) else
                       "000000000000010000000000" when (stat_led_en = '1') and (led_seq = 10) else
                       "000100000000000000000000" when (stat_led_en = '1') and (led_seq = 11) else
                       "010000000000000000000000" when (stat_led_en = '1') and (led_seq = 12) else
                       "000000000010000000000000" when (stat_led_en = '1') and (led_seq = 13) else
                       "000000001000000000000000" when (stat_led_en = '1') and (led_seq = 14) else
                       "000000100000000000000000" when (stat_led_en = '1') and (led_seq = 15) else
                       "000010000000000000000000" when (stat_led_en = '1') and (led_seq = 16) else
                       "000000000000000010000000" when (stat_led_en = '1') and (led_seq = 17) else
                       "000000000000000000100000" when (stat_led_en = '1') and (led_seq = 18) else
                       "000000000000000000001000" when (stat_led_en = '1') and (led_seq = 19) else
                       "000000000000000000000010" when (stat_led_en = '1') and (led_seq = 20) else
                       "000000000000001000000000" when (stat_led_en = '1') and (led_seq = 21) else
                       "000000000000100000000000" when (stat_led_en = '1') and (led_seq = 22) else
                       "001000000000000000000000" when (stat_led_en = '1') and (led_seq = 23) else
                       "100000000000000000000000" when (stat_led_en = '1') and (led_seq = 24) else
                       "000000000000000000000000";

  -- Sequence the front-panel LEDs based on the sequence counter
  pulse_led_seq <= "000001" when (pulse_led_en = '1') and (led_seq =  1) else
                   "000010" when (pulse_led_en = '1') and (led_seq =  2) else
                   "000100" when (pulse_led_en = '1') and (led_seq =  3) else
                   "001000" when (pulse_led_en = '1') and (led_seq =  4) else
                   "010000" when (pulse_led_en = '1') and (led_seq =  5) else
                   "100000" when (pulse_led_en = '1') and (led_seq =  6) else
                   "000001" when (pulse_led_en = '1') and (led_seq =  7) else
                   "000010" when (pulse_led_en = '1') and (led_seq =  8) else
                   "000100" when (pulse_led_en = '1') and (led_seq =  9) else
                   "001000" when (pulse_led_en = '1') and (led_seq = 10) else
                   "010000" when (pulse_led_en = '1') and (led_seq = 11) else
                   "100000" when (pulse_led_en = '1') and (led_seq = 12) else
                   "000001" when (pulse_led_en = '1') and (led_seq = 13) else
                   "000010" when (pulse_led_en = '1') and (led_seq = 14) else
                   "000100" when (pulse_led_en = '1') and (led_seq = 15) else
                   "001000" when (pulse_led_en = '1') and (led_seq = 16) else
                   "010000" when (pulse_led_en = '1') and (led_seq = 17) else
                   "100000" when (pulse_led_en = '1') and (led_seq = 18) else
                   "000001" when (pulse_led_en = '1') and (led_seq = 19) else
                   "000010" when (pulse_led_en = '1') and (led_seq = 20) else
                   "000100" when (pulse_led_en = '1') and (led_seq = 21) else
                   "001000" when (pulse_led_en = '1') and (led_seq = 22) else
                   "010000" when (pulse_led_en = '1') and (led_seq = 23) else
                   "100000" when (pulse_led_en = '1') and (led_seq = 24) else
                   "000000";

  -- Then, we instantiate the LED controller and control it via the LED state
  -- vector.
  cmp_bicolor_led_ctrl : gc_bicolor_led_ctrl
    generic map
    (
      g_NB_COLUMN    => 6,
      g_NB_LINE      => 2,
      g_clk_freq     => 20000000,
      g_refresh_rate => 250
    )
    port map
    (
      clk_i           => clk_20_i,
      rst_n_i         => rst_20_n,
      led_intensity_i => "1111111",
      led_state_i     => bicolor_led_state,
      column_o(0)     => led_ttl_i2c_o,
      column_o(1)     => led_oterm_wr_o,
      column_o(2)     => led_iterm_syserror_o,
      column_o(3)     => led_gf_syspw_o,
      column_o(4)     => led_gp_2_4_o,
      column_o(5)     => led_gp_1_3_o,
      line_o(0)       => led_ctrl0_o,
      line_o(1)       => led_ctrl1_o,
      line_oen_o(0)   => led_ctrl0_oen_o,
      line_oen_o(1)   => led_ctrl1_oen_o
    );

  --============================================================================
  -- Drive pulse LEDs
  --============================================================================
  led_front_n_o <= not pulse_led_seq;
  led_rear_n_o  <= not pulse_led_seq;

  --============================================================================
  -- Drive unused outputs with safe values
  --============================================================================
  ttl_o <= (others => '0');
  inv_o <= (others => '0');
  rs485_o <= (others => '1');
  flash_cs_n_o <= '1';
  flash_mosi_o <= '0';
  flash_sclk_o <= '0';

    -- Channel enable
    global_oen_o           <= '0';
    ttl_oen_o              <= '0';
    inv_oen_o              <= '0';
    rs485_oen_o            <= '0';

  -- DAC outputs: enables to '1' (disable DAC comm interface) and SCK, DIN to '0'
  dac20_sync_n_o  <= '1';
  dac20_din_o     <= '0';
  dac20_sclk_o    <= '0';
  dac125_sync_n_o <= '1';
  dac125_din_o    <= '0';
  dac125_sclk_o   <= '0';

  -- SFP lines all open-drain, set to high-impedance
  sfp_rate_select_o <= 'Z';
  sfp_mod_def1_b    <= 'Z';
  sfp_mod_def2_b    <= 'Z';
  sfp_tx_disable_o  <= 'Z';

end architecture arch;
--==============================================================================
--  architecture end
--==============================================================================
